CentralSymmetryTest <- function(X,
                                adj.method = "holm", # c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none")
                                marg.skew.package = "lawstat", # psych
                                marg.option = "MGG", # c("MGG", "CM", "M"),
                                marg.boot = TRUE,
                                marg.B = 1000,
                                marg.q = 8/9,
                                copula.B = 1000){

  d <- ncol(X)
  adj.pvalue <- pvalue <- numeric(d+1)

  # central symmetry on each marginal distribution

  for(j in 1:d){

    if(marg.skew.package == "lawstat"){
      pvalue[j] <- lawstat::symmetry.test(
        X[,j],
        option = marg.option,
        side = "both",
        boot = marg.boot,
        B = marg.B,
        q = marg.q)$p.value
    }

    if(marg.skew.package == "psych"){
      pvalue[j] <- psych::mardia(x = X[,j], na.rm = TRUE, plot = FALSE)$p.skew
    }

  }

  # Central (Radial) Symmetry on the copula

  pvalue[d+1] <- copula::radSymTest(X, N = copula.B)$p.value

  # Adjusted p-values

  adj.pvalue <- p.adjust(p = pvalue, method = adj.method)

  nomi <- c(paste("X.",1:d,sep=""),"Copula")

  names(pvalue) <- names(adj.pvalue) <- nomi

  return(list(
    raw.pvalue = pvalue,
    adj.pvalue = adj.pvalue,
    overall.adj.pvalue = min(adj.pvalue)))

}
