# https://www.researchgate.net/post/Does_there_exist_test_statistic_for_testing_of_the_significance_of_difference_between_the_kurtosis_of_the_two_samples

HomoKurtosesTest <- function(X,
                             B = 1000, # number of bootstrap replications
                             kurt.package = "e1071" # PerformanceAnalytics (sd non corretta), e1071 (sd corretta), moments (sd non corretta)
                             ){

  n <- nrow(X)
  d <- ncol(X)
  vecX <- as.vector(X)

  # ---------------------- #
  # On the observed sample #
  # ---------------------- #

  obs.kurt <- numeric(d)

  for(j in 1:d){
    if(kurt.package == "e1071") # excess kurtosis (unbiased sd)
      obs.kurt[j] <- e1071::kurtosis(X[,j])
    if(kurt.package == "moments") # kurtosis (biased sd)
      obs.kurt[j] <- moments::kurtosis(X[,j])
    if(kurt.package == "PerformanceAnalytics") # excess kurtosis (biased sd)
      obs.kurt[j] <- PerformanceAnalytics::kurtosis(X[,j], method = "excess")
  }

  # test statistic

  abs.obs.diff <- abs(outer(obs.kurt,obs.kurt,'-'))
  obs.dist <- abs.obs.diff[lower.tri(abs.obs.diff)]
  obs.T <- mean(obs.dist)

  # ----------------------- #
  # On the bootstrap sample #
  # ----------------------- #

  boot.kurt <- list()
  boot.T <- numeric(B)

  for(b in 1:B){

    boot.X <- matrix(sample(vecX, size = n*d, replace = FALSE), nrow = n, ncol = d)

    boot.kurt[[b]] <- numeric(d)

    for(j in 1:d){
      if(kurt.package == "e1071")
        boot.kurt[[b]][j] <- e1071::kurtosis(boot.X[,j])
      if(kurt.package == "moments")
        boot.kurt[[b]][j] <- moments::kurtosis(boot.X[,j])
      if(kurt.package == "PerformanceAnalytics")
        boot.kurt[[b]][j] <- PerformanceAnalytics::kurtosis(boot.X[,j], method = "excess")
    }

    # test statistic

    abs.boot.diff <- abs(outer(boot.kurt[[b]],boot.kurt[[b]],'-'))
    boot.dist     <- abs.boot.diff[lower.tri(abs.boot.diff)]
    boot.T[b]     <- mean(boot.dist)

  }

  Boot.CDF.T <- ecdf(boot.T)
  p.value    <- 1-Boot.CDF.T(obs.T)

  return(p.value)

}

